@@ -506,401 +506,6 @@ void set_f2int_pars(double *XtX, double *ytX, double *tau, int *n, int *p, int *
 
 
 
-double LogFactorial(int);
-
-/* Get the next n-tuple in lexicographical order with values 0, 1, ..., base-1.
-   Return true if successful, false if unable to increment further. */
-int GetNextTuple(int* tuple, int n, int base)
-{
-  int j = 0;
-  while (j < n && tuple[j] == base-1)
-    {
-      tuple[j] = 0;
-      j++;
-    }
-  if (j < n)
-    tuple[j]++;
-  return (j < n);
-}
-
-int BinomialCoefficient(int power, int nu) {
-  int ans;
-  /* power can only be 2 or 4 */
-  /* 0 <= nu <= power */
-  if (power == 2) {
-    ans= 1 + nu%2;
-  } else if (power == 4) {
-      if (nu == 0 || nu == 4)
-	ans= 1;
-      if (nu == 1 || nu == 3)
-	ans= 4;
-      if (nu == 2)
-	ans= 6;
-  } else ans= 0; /* should never happen */
-  return ans;
-}
-
-/* NB: The equation in the paper is off by one.
-   The following code, taken from the paper's definition of kappa,
-   acually computes kappa + 1. */
-double one_plus_kappa
-(
- double dof, /* degrees of freedom */
- int r/* summation index */
- )
-{
-  double product = 1.0;
-  int i;
-
-  if (r == 0)
-    return 1.0;
-
-  for (i = 1; i <= r; i++)
-    product *= 0.5*dof - i;
-  return pow(0.5*dof- 1.0, r) / product;
-}
-
-
-/*
-    Expectation of prod (x_i)^(2*power), where x_i ~ multivariate T_dof(mu,sigma). Set dof= -1 for x_i ~ N(mu,sigma)
-    mu    = mean, a vector of length n
-    sigma = covariance, n by n matrix
-    dof   = degrees of freedom, -1 signals normal case
-    power = 2 or 4, exponent on components
-    Note: This function was written by John D. Cook
-*/
-double mvtexpect(const double* mu, const double* sigma, int n, int power, double dof)
-{
-  double product = 1.0;
-  double sum = 0.0;
-  double covariance_term, mean_term, temp;
-  int index_sum;
-
-  int s = power*n;
-  int half_s = s/2; 
-  int j, k, r, half_power;
-
-  int* nu_index; /* indices nu_0 through nu_{n-1} */
-  nu_index = ivector(0,n);
-
-  for (r = 0; r <= half_s; r++)
-    {
-
-      for (j = 0; j < n; j++)
-	nu_index[j] = 0;
-
-      do
-	{
-	  product = 1.0;
-
-	  index_sum = 0;
-	  for (j = 0; j < n; j++)
-	    index_sum += nu_index[j];
-	  if (index_sum % 2)
-	    product *= -1;
-
-	  for (j = 0; j < n; j++)
-	    product *= BinomialCoefficient(power, nu_index[j]); 
-
-	  if (dof > 0.0)
-	    product *= one_plus_kappa(dof, r);
-	  /* else normal case and multiplicative term is 1 */
-
-	  covariance_term = 0.0;
-	  half_power = power/2;
-	  for (j = 0; j < n; j++)
-	    {
-	      temp = 0.0; /* double_sum accumulates the product sigma*h */
-	      for (k = 0; k < n; k++)
-		{
-		  /* sigma is stored by columns, so sigma_{ij} = sigma[i + j*n] */
-		  temp += sigma[j + k*n]*(half_power - nu_index[k]);
-		}
-	      covariance_term += (half_power - nu_index[j])*temp;
-	    }
-	  product *= pow(0.5*covariance_term, r);
-
-	  mean_term = 0.0;
-	  for (j = 0; j < n; j++)
-	    mean_term += (half_power - nu_index[j])*mu[j];
-	  product *= pow(mean_term, s - 2*r); 
-	  product /= exp(LogFactorial(r) + LogFactorial(s - 2*r));
-
-	  sum += product;
-
-	} while( GetNextTuple(nu_index, n, power+1) );
-    }
-  free_ivector(nu_index,0,n);
-  return sum;
-}
-
-double LogFactorial(int n) {
-  double ans;
-  if (n > 254) {
-      double x = n + 1;
-      ans= (x - 0.5)*log(x) - x + 0.5*log(2*M_PI) + 1.0/(12.0*x);
-  } else {
-        double lf[] = {
-            0.000000000000000,
-            0.000000000000000,
-            0.693147180559945,
-            1.791759469228055,
-            3.178053830347946,
-            4.787491742782046,
-            6.579251212010101,
-            8.525161361065415,
-            10.604602902745251,
-            12.801827480081469,
-            15.104412573075516,
-            17.502307845873887,
-            19.987214495661885,
-            22.552163853123421,
-            25.191221182738683,
-            27.899271383840894,
-            30.671860106080675,
-            33.505073450136891,
-            36.395445208033053,
-            39.339884187199495,
-            42.335616460753485,
-            45.380138898476908,
-            48.471181351835227,
-            51.606675567764377,
-            54.784729398112319,
-            58.003605222980518,
-            61.261701761002001,
-            64.557538627006323,
-            67.889743137181526,
-            71.257038967168000,
-            74.658236348830158,
-            78.092223553315307,
-            81.557959456115029,
-            85.054467017581516,
-            88.580827542197682,
-            92.136175603687079,
-            95.719694542143202,
-            99.330612454787428,
-            102.968198614513810,
-            106.631760260643450,
-            110.320639714757390,
-            114.034211781461690,
-            117.771881399745060,
-            121.533081515438640,
-            125.317271149356880,
-            129.123933639127240,
-            132.952575035616290,
-            136.802722637326350,
-            140.673923648234250,
-            144.565743946344900,
-            148.477766951773020,
-            152.409592584497350,
-            156.360836303078800,
-            160.331128216630930,
-            164.320112263195170,
-            168.327445448427650,
-            172.352797139162820,
-            176.395848406997370,
-            180.456291417543780,
-            184.533828861449510,
-            188.628173423671600,
-            192.739047287844900,
-            196.866181672889980,
-            201.009316399281570,
-            205.168199482641200,
-            209.342586752536820,
-            213.532241494563270,
-            217.736934113954250,
-            221.956441819130360,
-            226.190548323727570,
-            230.439043565776930,
-            234.701723442818260,
-            238.978389561834350,
-            243.268849002982730,
-            247.572914096186910,
-            251.890402209723190,
-            256.221135550009480,
-            260.564940971863220,
-            264.921649798552780,
-            269.291097651019810,
-            273.673124285693690,
-            278.067573440366120,
-            282.474292687630400,
-            286.893133295426990,
-            291.323950094270290,
-            295.766601350760600,
-            300.220948647014100,
-            304.686856765668720,
-            309.164193580146900,
-            313.652829949878990,
-            318.152639620209300,
-            322.663499126726210,
-            327.185287703775200,
-            331.717887196928470,
-            336.261181979198450,
-            340.815058870798960,
-            345.379407062266860,
-            349.954118040770250,
-            354.539085519440790,
-            359.134205369575340,
-            363.739375555563470,
-            368.354496072404690,
-            372.979468885689020,
-            377.614197873918670,
-            382.258588773060010,
-            386.912549123217560,
-            391.575988217329610,
-            396.248817051791490,
-            400.930948278915760,
-            405.622296161144900,
-            410.322776526937280,
-            415.032306728249580,
-            419.750805599544780,
-            424.478193418257090,
-            429.214391866651570,
-            433.959323995014870,
-            438.712914186121170,
-            443.475088120918940,
-            448.245772745384610,
-            453.024896238496130,
-            457.812387981278110,
-            462.608178526874890,
-            467.412199571608080,
-            472.224383926980520,
-            477.044665492585580,
-            481.872979229887900,
-            486.709261136839360,
-            491.553448223298010,
-            496.405478487217580,
-            501.265290891579240,
-            506.132825342034830,
-            511.008022665236070,
-            515.890824587822520,
-            520.781173716044240,
-            525.679013515995050,
-            530.584288294433580,
-            535.496943180169520,
-            540.416924105997740,
-            545.344177791154950,
-            550.278651724285620,
-            555.220294146894960,
-            560.169054037273100,
-            565.124881094874350,
-            570.087725725134190,
-            575.057539024710200,
-            580.034272767130800,
-            585.017879388839220,
-            590.008311975617860,
-            595.005524249382010,
-            600.009470555327430,
-            605.020105849423770,
-            610.037385686238740,
-            615.061266207084940,
-            620.091704128477430,
-            625.128656730891070,
-            630.172081847810200,
-            635.221937855059760,
-            640.278183660408100,
-            645.340778693435030,
-            650.409682895655240,
-            655.484856710889060,
-            660.566261075873510,
-            665.653857411105950,
-            670.747607611912710,
-            675.847474039736880,
-            680.953419513637530,
-            686.065407301994010,
-            691.183401114410800,
-            696.307365093814040,
-            701.437263808737160,
-            706.573062245787470,
-            711.714725802289990,
-            716.862220279103440,
-            722.015511873601330,
-            727.174567172815840,
-            732.339353146739310,
-            737.509837141777440,
-            742.685986874351220,
-            747.867770424643370,
-            753.055156230484160,
-            758.248113081374300,
-            763.446610112640200,
-            768.650616799717000,
-            773.860102952558460,
-            779.075038710167410,
-            784.295394535245690,
-            789.521141208958970,
-            794.752249825813460,
-            799.988691788643450,
-            805.230438803703120,
-            810.477462875863580,
-            815.729736303910160,
-            820.987231675937890,
-            826.249921864842800,
-            831.517780023906310,
-            836.790779582469900,
-            842.068894241700490,
-            847.352097970438420,
-            852.640365001133090,
-            857.933669825857460,
-            863.231987192405430,
-            868.535292100464630,
-            873.843559797865740,
-            879.156765776907600,
-            884.474885770751830,
-            889.797895749890240,
-            895.125771918679900,
-            900.458490711945270,
-            905.796028791646340,
-            911.138363043611210,
-            916.485470574328820,
-            921.837328707804890,
-            927.193914982476710,
-            932.555207148186240,
-            937.921183163208070,
-            943.291821191335660,
-            948.667099599019820,
-            954.046996952560450,
-            959.431492015349480,
-            964.820563745165940,
-            970.214191291518320,
-            975.612353993036210,
-            981.015031374908400,
-            986.422203146368590,
-            991.833849198223450,
-            997.249949600427840,
-            1002.670484599700300,
-            1008.095434617181700,
-            1013.524780246136200,
-            1018.958502249690200,
-            1024.396581558613400,
-            1029.838999269135500,
-            1035.285736640801600,
-            1040.736775094367400,
-            1046.192096209724900,
-            1051.651681723869200,
-            1057.115513528895000,
-            1062.583573670030100,
-            1068.055844343701400,
-            1073.532307895632800,
-            1079.012946818975000,
-            1084.497743752465600,
-            1089.986681478622400,
-            1095.479742921962700,
-            1100.976911147256000,
-            1106.478169357800900,
-            1111.983500893733000,
-            1117.492889230361000,
-            1123.006317976526100,
-            1128.523770872990800,
-            1134.045231790853000,
-            1139.570684729984800,
-            1145.100113817496100,
-            1150.633503306223700,
-            1156.170837573242400,
-	  };
-        ans= lf[n];
-  }
-  return ans;
-}
 
 
 
@@ -956,7 +561,7 @@ void modelSelectionGibbs(int *postSample, double *postOther, double *margpp, int
   if (*niter >10) { niter10= *niter/10; } else { niter10= 1; }
   for (j=0; j< *(*pars).p; j++) { margpp[j]= 0; }
   nsel= *ndeltaini;
-  for (j=0; j< nsel; j++) { sel[j]= deltaini[j]; postSample[deltaini[j]*niterthin]= postMode[deltaini[j]]= 1; }
+  for (j=0; j< nsel; j++) { sel[j]= deltaini[j]; postMode[deltaini[j]]= 1; }
   if ((*prDelta)==2) { postOther[0]= *(*pars).prDeltap; }
   currentJ= marginalFunction(sel,&nsel,pars) + priorFunction(sel,&nsel,pars);
   postProb[0]= *postModeProb= currentJ;
@@ -1017,7 +622,7 @@ void modelSelectionGibbs2(int *postSample, double *postOther, double *margpp, in
   if (*niter >10) { niter10= *niter/10; } else { niter10= 1; }
   for (j=0; j< *(*pars).p; j++) { margpp[j]= 0; }
   nsel= *ndeltaini;
-  for (j=0; j< nsel; j++) { sel[j]= deltaini[j]; postSample[deltaini[j]*niterthin]= postMode[deltaini[j]]= 1; }
+  for (j=0; j< nsel; j++) { sel[j]= deltaini[j]; postMode[deltaini[j]]= 1; }
   if ((*prDelta)==2) { postOther[0]= *(*pars).prDeltap; }
   currentJ= integrals->getJoint(sel,&nsel,pars);
   postProb[0]= *postModeProb= currentJ;
@@ -1176,8 +781,8 @@ void fppmomNegC_non0(double **ans, double *th, double **S, double *phi, double *
 }
 
 void momIntegralApproxC(double *ILaplace, double *thopt, double **Voptinv, double *fopt, int *n, int *nsel, double *m, double **S, double *detS, double *phi, double *tau, int *r, int *logscale) {
-  int i, emptyint, iter, maxit=50;
-  double emptydouble, ftol= 1.0e-2, **Vopt, detVopt, **dirth;
+  int i, emptyint, iter, maxit=100;
+  double emptydouble, ftol= 1.0e-5, **Vopt, detVopt, **dirth;
 
   Vopt= dmatrix(1,*nsel,1,*nsel); dirth= dmatrix(1,*nsel,1,*nsel);
   set_f2opt_pars(m,S,&emptydouble,&emptydouble,&emptydouble,&emptydouble,&emptydouble,phi,tau,r,n,nsel,&emptyint,nsel);
@@ -1252,7 +857,7 @@ double MC_mom_T(double *m,double **Sinv,int *nu,int *r,int *nsel, int *B) {
 // - phi: residual variance
 // - tau: prior dispersion parameter
 // - r: MOM power parameter
-// - method==0 for Laplace; method==1 for Monte Carlo; method==2 for plug-in
+// - method==0 for Laplace; method==1 for Monte Carlo; method==2 for plug-in; method== -1 for exact calculation if p<20
 // - B: number of Monte Carlo samples. Ignored unless method==1.
 // - logscale: if set to 1 result is returned in log scale
 
@@ -1295,7 +900,7 @@ double pmomMarginalKC(int *sel, int *nsel, struct marginalPars *pars) {
 
     num= -.5*(*(*pars).sumy2 - quadratic_xtAx(m,S,1,*nsel))/(*(*pars).phi);
     den= .5*((*(*pars).n +.0)*(LOG_M_2PI+logphi) + log(detS) + (*nsel)*logtau) + (*nsel)*(*(*pars).r)*(logtau+logphi+ldoublefact(2*(*(*pars).r)-1));
-    if (*(*pars).method ==0) { //Laplace
+    if ((*(*pars).method ==0) | ((*(*pars).method == -1) & (*nsel)>10))  { //Laplace
       thopt= dvector(1,*nsel); Voptinv= dmatrix(1,*nsel,1,*nsel);
       momIntegralApproxC(&ans,thopt,Voptinv,&fopt,(*pars).n,nsel,m,S,&detS,(*pars).phi,(*pars).tau,(*pars).r,(*pars).logscale);
       free_dvector(thopt,1,*nsel); free_dmatrix(Voptinv,1,*nsel,1,*nsel);
@@ -1304,6 +909,11 @@ double pmomMarginalKC(int *sel, int *nsel, struct marginalPars *pars) {
       ans= MC_mom_normal(m,Sinv,(*pars).r,nsel,(*pars).B);
     } else if (*(*pars).method ==2) { //Plug-in
       ans= rsumlogsq(m,(*pars).r,nsel);
+    } else if ((*(*pars).method == -1) & (*nsel)<=10) { //Exact
+      Voptinv= dmatrix(1,*nsel,1,*nsel);
+      for (i=1; i<= *nsel; i++) for (j=i; j<= *nsel; j++) Voptinv[i][j]= Voptinv[j][i]= Sinv[i][j] * (*(*pars).phi);
+      ans= log(mvtexpect(m, Voptinv, *nsel, 2, -1));
+      free_dmatrix(Voptinv,1,*nsel,1,*nsel);
     }
     ans+= num - den;
     free_dvector(m,1,*nsel);
@@ -1330,7 +940,7 @@ SEXP pmomMarginalUI(SEXP Ssel, SEXP Snsel, SEXP Sn, SEXP Sp, SEXP Sy, SEXP Ssumy
 
 double pmomMarginalUC(int *sel, int *nsel, struct marginalPars *pars) {
   int i, j, nu;
-  double num, den, ans=0.0, term1, *m, **S, **Sinv, detS, *thopt, **Voptinv, fopt, phiadj, tauinv= 1.0/(*(*pars).tau), nuhalf, alphahalf=.5*(*(*pars).alpha), lambdahalf=.5*(*(*pars).lambda);
+  double num, den, ans=0.0, term1, *m, **S, **Sinv, detS, *thopt, **Voptinv, fopt, phiadj, tauinv= 1.0/(*(*pars).tau), nuhalf, alphahalf=.5*(*(*pars).alpha), lambdahalf=.5*(*(*pars).lambda), ss;
   if (*nsel ==0) {
     term1= .5*(*(*pars).n + *(*pars).alpha);
     num= .5*(*(*pars).alpha)*log(*(*pars).lambda) + gamln(&term1);
@@ -1344,9 +954,10 @@ double pmomMarginalUC(int *sel, int *nsel, struct marginalPars *pars) {
     nuhalf= (*(*pars).r)*(*nsel) + .5*(*(*pars).n + *(*pars).alpha);
     nu= (int) (2.0*nuhalf);
 
-    num= gamln(&nuhalf) + alphahalf*log(lambdahalf) + nuhalf*(log(2.0) - log(*(*pars).lambda + *(*pars).sumy2 - quadratic_xtAx(m,S,1,*nsel)));
+    ss= *(*pars).lambda + *(*pars).sumy2 - quadratic_xtAx(m,S,1,*nsel);
+    num= gamln(&nuhalf) + alphahalf*log(lambdahalf) + nuhalf*(log(2.0) - log(ss));
     den= (*nsel)*ldoublefact(2*(*(*pars).r)-1.0) + .5*(*(*pars).n * LOG_M_2PI + log(detS)) + (*nsel)*(.5 + *(*pars).r)*log(*(*pars).tau) + gamln(&alphahalf);
-    if (*(*pars).method ==0) { //Laplace
+    if ((*(*pars).method ==0) | ((*(*pars).method == -1) & (*nsel)>10))  { //Laplace
       thopt= dvector(1,*nsel); Voptinv= dmatrix(1,*nsel,1,*nsel);
       phiadj= (nu+.0)/(nu-2.0);
       momIntegralApproxC(&ans,thopt,Voptinv,&fopt,(*pars).n,nsel,m,S,&detS,&phiadj,(*pars).tau,(*pars).r,(*pars).logscale);
@@ -1357,6 +968,11 @@ double pmomMarginalUC(int *sel, int *nsel, struct marginalPars *pars) {
       ans= MC_mom_T(m,Sinv,&nu,(*pars).r,nsel,(*pars).B);
     } else if (*(*pars).method ==2) {  //Plug-in
       ans= rsumlogsq(m,(*pars).r,nsel);
+    } else if ((*(*pars).method == -1) & (*nsel)<=10) { //Exact
+      Voptinv= dmatrix(1,*nsel,1,*nsel);
+      for (i=1; i<= *nsel; i++) for (j=i; j<= *nsel; j++) Voptinv[i][j]= Voptinv[j][i]= Sinv[i][j] * ss / (nu+.0);
+      ans= log(mvtexpect(m, Voptinv, *nsel, 2, nu));
+      free_dmatrix(Voptinv,1,*nsel,1,*nsel);
     }
     ans+= num - den;
     free_dvector(m,1,*nsel); free_dmatrix(S,1,*nsel,1,*nsel); free_dmatrix(Sinv,1,*nsel,1,*nsel);
@@ -1440,8 +1056,8 @@ void fppimomNegC_non0(double **ans, double *th, double *XtX, double *ytX, double
 
 
 void imomIntegralApproxC(double *ILaplace, double *thopt, double **Voptinv, double *fopt, int *sel, int *nsel, int *n, int *p, double *XtX, double *ytX, double *phi, double *tau, int *logscale) {
-  int iter, maxit=50, emptyint;
-  double **V, **Vinv, ftol= 1.0e-2, **dirth, **Vopt, detVopt, emptydouble=0, **emptymatrix;
+  int iter, maxit=100, emptyint;
+  double **V, **Vinv, ftol= 1.0e-5, **dirth, **Vopt, detVopt, emptydouble=0, **emptymatrix;
 
   V= dmatrix(1,*nsel,1,*nsel); Vinv= dmatrix(1,*nsel,1,*nsel); Vopt= dmatrix(1,*nsel,1,*nsel); dirth= dmatrix(1,*nsel,1,*nsel);
   emptymatrix= dmatrix(1,1,1,1);
@@ -1621,8 +1237,8 @@ void fppimomUNegC_non0(double **ans, double *th, double *sumy2, double *XtX, dou
 
 
 void imomUIntegralApproxC(double *ILaplace, double *thopt, int *sel, int *nsel, int *n, int *p, double *sumy2, double *XtX, double *ytX, double *alpha, double *lambda, double *tau, int *logscale) {
-  int iter, maxit=50, emptyint;
-  double ftol= 1.0e-4, **dirth, **Vopt, **Voptinv, detVopt, emptydouble=0, **emptymatrix, fopt;
+  int iter, maxit=100, emptyint;
+  double ftol= 1.0e-10, **dirth, **Vopt, **Voptinv, detVopt, emptydouble=0, **emptymatrix, fopt;
 
   Vopt= dmatrix(1,*nsel +1,1,*nsel +1); Voptinv= dmatrix(1,*nsel +1,1,*nsel +1);
   dirth= dmatrix(1,*nsel +1,1,*nsel +1);
